comment{
;Check if shot is possible; if so, take the shot
;[shooter, [target x,y,z], <shoot?>, <stance>, <magazine>, <dispersion override>] call GENB_IIF_fire
;<shoot?> - If false, unit will not shoot (used when you just want to check if shot is possible). Default true.
;<stance> - "up", "down", "auto" (depends on combatmode). Def "auto"
;<magazine> - mag to use. By default, choose a mag he has that is configured.
;<dispersion override> - use this dispersion [min, max] instead of what is configured for weapon

;RETURNS (only 1st 2 are returned if shoot == false):

;"OK" - can target & shoot
;"NOTARGET" - can't target location
;"NOSHOOT" - can't shoot (in vehicle, dead, out of ammo, no weapon capable of IF, etc)
;"BUSY" - busy w/another IF shot
};


private ["_return","_shooter","_shoot","_mag","_data"];

_return = "OK";
_shooter = _this select 0;

_shoot = true;
if(count _this > 2) then {_shoot = _this select 2};

_mag = "";
if(count _this > 4) then {_mag = _this select 4};
	comment{IF NO MAGAZINE SPECIFIED, AUTODETECT WHICH CONFIGURED MAG UNIT HAS};
	if(_mag == "") then
	{
		private "_tmp"; _tmp = magazines _shooter; 
		{if((_x select 0) in _tmp) then {_mag = _x select 0};} foreach (GENB_IIF_data select 2);
		if(_mag == "") then {_mag = ((GENB_IIF_data select 2) select 0) select 0};
	};

_data = []; {if(_mag == (_x select 0)) then {_data = +_x};} foreach (GENB_IIF_data select 2);

comment{PRELIMINARY CHECK IN CASE SHOT CALCULATION ISN'T REQUIRED};
if(_shoot) then
{
	if( !(alive _shooter) || !(canfire _shooter) || (vehicle _shooter != _shooter) || (({(_shooter ammo _x) > 0} count (_data select 1)) == 0) ) then {_return = "NOSHOOT"};
	if(_shooter in (GENB_IIF_data select 0)) then {_return = "BUSY"};
};

comment{
;EXTRACT DATA AND CALCULATE SHOT (only if required)
;Randomize target location based on dispersion & skill

;dir = direction to target
;x = lateral distance to target
;y = elevation difference between shooter & target
;z = required elevation for invis target (equation found in data_model.sqs)
;(GENB_IIF_data select 1) select 0) is a function to find height ASL
};

if(_return == "OK") then
{
	private ["_stance","_target","_disp","_mod","_pos","_dir","_x","_y","_z","_targX","_targY","_targZ","_shooterZ"];

	_target = _this select 1;	

	_stance = "auto";
	if(count _this > 3) then {if((_this select 3) in ["up","down","auto"]) then {_stance = _this select 3}};
	if(_stance == "auto") then {if ((behaviour _shooter == "COMBAT") || (behaviour _shooter == "STEALTH") || !(canstand _shooter)) then {_stance = "down"} else {_stance = "up"};};

	_disp = _data select 2;
	if(count _this > 5) then {_disp = _this select 5};
	_disp = (((_disp select 1) - (_disp select 0)) * (1 - skill _shooter)) + (_disp select 0);

	if(_stance == "down") then {_data = (_data select 3) select 0};
	if(_stance == "up") then {_data = (_data select 3) select 1};

	_pos = getpos _shooter;
	_shooterZ = _pos call (GENB_IIF_data select 1);

	_dir = ((_target select 0) - (_pos select 0)) atan2 ((_target select 1) - (_pos select 1));
	_mod = _disp * random 12;
	_dir = _dir - _mod + (2*random _mod);

	_x = sqrt(((_pos select 0)-(_target select 0))^2 + ((_pos select 1)-(_target select 1))^2);
	_mod = (_disp * random 0.33) * _x;
	_x = _x - _mod + (2*random _mod);

	_y = (_target call (GENB_IIF_data select 1)) - _shooterZ;
	_z = (_data select 0) call (_data select 1);

	_targX = (_pos select 0) + ((sin _dir)*50);
	_targY = (_pos select 1) + ((cos _dir)*50);

	_targZ = [_targX, _targY] call (GENB_IIF_data select 1);
	_targZ = (_shooterZ - _targZ) + _z;

	comment{CHECK IF SHOT IS POSSIBLE, TAKE IF REQUIRED.};

	if((_z > 50) || (_targZ < 0) || (_x < 15)) then {_return = "NOTARGET"};
	if((_shoot) && (_return == "OK")) then {[_shooter, _stance, _mag, [_targX,_targY,_targZ]] exec "inf_IF\fireAtTarget.sqs"};
};

_return

