comment{
;9 April 2006

;All weapons defined in if_IF.cfg are run thru this function at the start of the mission.
;It returns the appropriate equation and constants for a specified muzzle velocity, for both standing and prone positions.

;Currently, equation is the same for all muzzle velocities (constants differ, but not the equation)
;However, storing it separately for each weapon allows for possible upgrades in the future
;(e.g., different eqns for diff velocities; inclusion of rockets or bullets)

;Storing this information in the weapons array eliminates needles re-calculation after mission start.

;Separating this data allows the underlying mathematical model to be changed by simply changing one file, making future upgrades simple.
};

private ["_vel","_return","_equation","_const_stand","_const_prone","_interpolate"];

_vel = _this select 3;

comment {This function is used to interpolate constants between pre-calculated muzzle velocities.
For example, constants for velocities of 80 and 85 are already known. This function will find a constant for velocity 82
by making a simple line between the two known constants, and finding the point 82 along that line. An imperfect method,
but MUCH simpler than making a data-fitting equation for every constant and position.};

_interpolate =
{
	private["_result","_data","_d1","_d2","_i","_c","_d"];
	_data = _this select 0;
	_result = 9E99;
	comment{Find two nearest data points to input velocity. Abort if input vel is a known vel. Data format: [ [constant, velocity], [...] ] --- NEED TO SPEED UP THIS SHITTY ALGORITHM???};
	_i = 0;
	_d1 = [[],9E99]; _d2 = [[],9E99];
	while{(_i < count _data) && (_result == 9E99)} do 
	{
		_c = _data select _i;
		_d = abs( (_c select 1) - _vel);
		if(_d == 0) then {_result = _c select 0} else
		{
			if(_d <= (_d1 select 1) ) then {_d2 = +_d1; _d1 = [_c,_d]} else {if(_d <= (_d2 select 1) ) then {_d2 = [_c,_d]}};
		};
		_i = _i + 1;
	};
	comment{Form line between data points and find target velocity (unless already known)};
	if(_result == 9E99) then 
	{
		_d1 = _d1 select 0; _d2 = _d2 select 0;
		private["_A","_Ax","_Ay","_B","_Bx","_By"];
		if((_d1 select 1) > (_d2 select 1)) then {_A = _d2; _B = _d1;} else {_A = _d1; _B = _d2;};
		_Ax = _A select 1; _Ay = _A select 0;
		_Bx = _B select 1; _By = _B select 0;
		_result = ((_By-_Ay)/(_Bx-_Ax))*_vel + (_Ay-((_By-_Ay)/(_Bx-_Ax))*_Ax);
	};
	_result
};

_equation = format
[
	{(%1) + (%2*_x^0*_y^1) + (%3*_x^0*_y^2) + (%4*_x^1*_y^0) + (%5*_x^1*_y^1) + (%6*_x^1*_y^2) + (%7*_x^2*_y^0) + (%8*_x^2*_y^1) + (%9*_x^2*_y^2) + (%10*_x^3*_y^0) + (%11*_x^3*_y^1) + (%12*_x^3*_y^2) + (%13*_x^4*_y^0) + (%14*_x^4*_y^1) + (%15*_x^4*_y^2)},
	"(_this select 0)","(_this select 1)","(_this select 2)","(_this select 3)","(_this select 4)","(_this select 5)","(_this select 6)","(_this select 7)","(_this select 8)","(_this select 9)","(_this select 10)","(_this select 11)","(_this select 12)","(_this select 13)","(_this select 14)"
];

_const_stand =
[
	[[[8.1738745731488094E-02,55],
	[7.3353839430988735E-02,60],
	[6.8481513227266019E-02,65],
	[5.4319228452946282E-02,70],
	[4.7380398839885894E-02,75],
	[4.6359165636932147E-02,80],
	[3.5739974359087372E-02,85],
	[3.2667907644354913E-02,90]],"(A)"] call _interpolate,

	[[[2.2919195805363572E+00,55],
	[2.3013212240922813E+00,60],
	[2.2394736033353762E+00,65],
	[2.0364428160567396E+00,70],
	[1.8947682178492384E+00,75],
	[1.8457758031576896E+00,80],
	[1.6137768731052144E+00,85],
	[1.5208845032161014E+00,90]],"(B)"] call _interpolate,

	[[[-2.4497512804531759E-03,55],
	[-6.5311155528999434E-03,60],
	[-7.5000514684079658E-03,65],
	[-6.6970282467394768E-03,70],
	[-6.0881410205234528E-03,75],
	[-6.3548015026566240E-03,80],
	[-4.6356279218923834E-03,85],
	[-4.0318358901307435E-03,90]],"(C)"] call _interpolate,

	[[[1.0034422141721611E-01,55],
	[2.3318796307489342E-02,60],
	[-4.9742830925841697E-02,65],
	[-5.4177330008104485E-02,70],
	[-5.4568036145043686E-02,75],
	[-6.8134572864468890E-02,80],
	[-6.7836694429297073E-02,85],
	[-7.7201199999997139E-02,90]],"(D)"] call _interpolate,

	[[[-3.7572042758240221E-02,55],
	[-3.2465650227554557E-02,60],
	[-2.6983414161188005E-02,65],
	[-2.1816610783305164E-02,70],
	[-1.8063974866988883E-02,75],
	[-1.6017023831703205E-02,80],
	[-1.2577901041473678E-02,85],
	[-1.0931964380977224E-02,90]],"(E)"] call _interpolate,

	[[[8.0330696897082941E-05,55],
	[1.1652437347858164E-04,60],
	[1.0581657669083306E-04,65],
	[8.3189807439457044E-05,70],
	[6.6288810805897872E-05,75],
	[6.2967418778177346E-05,80],
	[4.1520167808741412E-05,85],
	[3.3825017469154482E-05,90]],"(F)"] call _interpolate,

	[[[-1.2534530044828514E-03,55],
	[1.9033876833254339E-04,60],
	[1.1261606967865921E-03,65],
	[1.1019610173971352E-03,70],
	[1.0268915997944778E-03,75],
	[1.0198272569673164E-03,80],
	[8.7196498813426780E-04,85],
	[8.2809627718662452E-04,90]],"(G)"] call _interpolate,

	[[[2.5502988640262783E-04,55],
	[1.8712278129725466E-04,60],
	[1.3250807722061183E-04,65],
	[9.5272578273864820E-05,70],
	[7.0094654799372205E-05,75],
	[5.6272796344936722E-05,80],
	[4.0040841492210592E-05,85],
	[3.2024530669116225E-05,90]],"(H)"] call _interpolate,

	[[[-4.3815784276953553E-07,55],
	[-5.4920173252001756E-07,60],
	[-4.3482106977984226E-07,65],
	[-3.0909508500469150E-07,70],
	[-2.2308351120297423E-07,75],
	[-1.9860039576110457E-07,80],
	[-1.1828916327833261E-07,85],
	[-9.0428162784527924E-08,90]],"(I)"] call _interpolate,

	[[[7.7340251013252720E-06,55],
	[-6.9214122738136500E-07,60],
	[-4.6544629995533795E-06,65],
	[-4.2033611168168932E-06,70],
	[-3.6092780397954469E-06,75],
	[-3.1933437371641509E-06,80],
	[-2.4655565402272277E-06,85],
	[-2.1131792481252267E-06,90]],"(J)"] call _interpolate,

	[[[-7.1912858902238504E-07,55],
	[-4.5974063954475818E-07,60],
	[-2.8396288892566789E-07,65],
	[-1.8345574405982779E-07,70],
	[-1.2077367830901201E-07,75],
	[-8.8405611697820678E-08,80],
	[-5.7343478127810442E-08,85],
	[-4.2384032843848422E-08,90]],"(K)"] call _interpolate,

	[[[7.2923257825216518E-10,55],
	[9.4493628050778304E-10,60],
	[6.8821005449177494E-10,65],
	[4.4802105543303465E-10,70],
	[2.9927326659230185E-10,75],
	[2.5374946253087551E-10,80],
	[1.3644412853371071E-10,85],
	[9.8154880488810794E-11,90]],"(L)"] call _interpolate,

	[[[-9.8629537598759191E-09,55],
	[3.2101843998209579E-09,60],
	[7.4457415455709099E-09,65],
	[5.7979483658830450E-09,70],
	[4.3998896537236678E-09,75],
	[3.4158831518383913E-09,80],
	[2.3827827747775905E-09,85],
	[1.8379439719060016E-09,90]],"(M)"] call _interpolate,

	[[[7.1812456820498363E-10,55],
	[4.1598207238369471E-10,60],
	[2.3279320226936543E-10,65],
	[1.3700828489061765E-10,70],
	[8.1704934465765400E-11,75],
	[5.4901390637613099E-11,80],
	[3.2621475181505635E-11,85],
	[2.2320039948991705E-11,90]],"(N)"] call _interpolate,

	[[[-2.1487558224168986E-13,55],
	[-4.7073326726395592E-13,60],
	[-3.3427752459634884E-13,65],
	[-2.0097601253799683E-13,70],
	[-1.2735607682126373E-13,75],
	[-1.0608913836935421E-13,80],
	[-5.0967706799306918E-14,85],
	[-3.4755978353228210E-14,90]],"(O)"] call _interpolate
];



_const_prone =
[
	[[[7.8727293954975880E-02,55],
	[7.2200266022613424E-02,60],
	[6.8103676884916323E-02,65],
	[5.4432416908257501E-02,70],
	[4.7966400891554163E-02,75],
	[4.4165516818078204E-02,80],
	[3.4626822488683401E-02,85],
	[3.3384436560524165E-02,90]],"(A)"] call _interpolate,

	[[[2.2440215139708264E+00,55],
	[2.2884138491705124E+00,60],
	[2.2701090553950287E+00,65],
	[2.0675363506945290E+00,70],
	[1.9470104512674491E+00,75],
	[1.8137091813387749E+00,80],
	[1.6144962911811727E+00,85],
	[1.5876750511974600E+00,90]],"(B)"] call _interpolate,

	[[[-1.4319241078893444E-03,55],
	[-5.7238725743082594E-03,60],
	[-7.7586274520173113E-03,65],
	[-7.0752877456334851E-03,70],
	[-6.7758051975568070E-03,75],
	[-5.9738565959403947E-03,80],
	[-4.6249984200757016E-03,85],
	[-4.7884184691577656E-03,90]],"(C)"] call _interpolate,

	[[[1.2515356167088265E-01,55],
	[3.5275162129231874E-02,60],
	[-2.5320210766073270E-02,65],
	[-5.7675787028500958E-02,70],
	[-5.5511749027809087E-02,75],
	[-7.9456564489774228E-02,80],
	[-7.4092869863669530E-02,85],
	[-5.7069737644293322E-02,90]],"(D)"] call _interpolate,

	[[[-3.6501317932366731E-02,55],
	[-3.2503470964216352E-02,60],
	[-2.8343406344529094E-02,65],
	[-2.2435700744841084E-02,70],
	[-1.9061807385501828E-02,75],
	[-1.6066695675254410E-02,80],
	[-1.3047996658372399E-02,85],
	[-1.1907754441728761E-02,90]],"(E)"] call _interpolate,

	[[[5.7860981027668387E-05,55],
	[1.0478935300694876E-04,60],
	[1.1396801176123629E-04,65],
	[9.0987096190728818E-05,70],
	[7.6315603039571814E-05,75],
	[6.1403347384085441E-05,80],
	[4.3804775341942724E-05,85],
	[4.1170814586251222E-05,90]],"(F)"] call _interpolate,

	[[[-1.6916367812859160E-03,55],
	[-9.4948195419537404E-05,60],
	[7.5272242246919412E-04,65],
	[1.0192753897144170E-03,70],
	[9.6392541601289300E-04,75],
	[1.0023982679564563E-03,80],
	[8.7158584058468196E-04,85],
	[7.2251964589829321E-04,90]],"(G)"] call _interpolate,

	[[[2.4727402626511347E-04,55],
	[1.8906298920771066E-04,60],
	[1.4287101761837961E-04,65],
	[9.8795153306187046E-05,70],
	[7.5016990709414287E-05,75],
	[5.7368107362175462E-05,80],
	[4.2415101048747847E-05,85],
	[3.5792363207721988E-05,90]],"(H)"] call _interpolate,

	[[[-3.0537635728693725E-07,55],
	[-4.9915236726629713E-07,60],
	[-4.8519444649198257E-07,65],
	[-3.5291025714307930E-07,70],
	[-2.6883097232677301E-07,75],
	[-1.9874758873146993E-07,80],
	[-1.2992381356472616E-07,85],
	[-1.1478046693061111E-07,90]],"(I)"] call _interpolate,

	[[[9.7971356157822026E-06,55],
	[7.3610513924492692E-07,60],
	[-3.1182704357679293E-06,65],
	[-3.7623464072543003E-06,70],
	[-3.3343191233436281E-06,75],
	[-3.0455352310060905E-06,80],
	[-2.4438552780132224E-06,85],
	[-1.9255295923778676E-06,90]],"(J)"] call _interpolate,

	[[[-6.9527868099108656E-07,55],
	[-4.6764362082153527E-07,60],
	[-3.1184661801908560E-07,65],
	[-1.9090395719882410E-07,70],
	[-1.3037611773643377E-07,75],
	[-9.1015916849736741E-08,80],
	[-6.1583852351463758E-08,85],
	[-4.8029196925598015E-08,90]],"(K)"] call _interpolate,

	[[[4.3892675412249824E-10,55],
	[8.5768035227270416E-10,60],
	[7.9096054878245290E-10,65],
	[5.3982033968763155E-10,70],
	[3.7904703301078358E-10,75],
	[2.5926185759978838E-10,80],
	[1.5546400589427742E-10,85],
	[1.3078097957819564E-10,90]],"(L)"] call _interpolate,

	[[[-1.2817203020236580E-08,55],
	[1.1114249138356684E-09,60],
	[5.5014092051025882E-09,65],
	[5.2260631283558558E-09,70],
	[4.0939658718124881E-09,75],
	[3.2434325936662270E-09,80],
	[2.3655332597936981E-09,85],
	[1.7280063388734085E-09,90]],"(M)"] call _interpolate,

	[[[6.9253505613500819E-10,55],
	[4.2366950628084246E-10,60],
	[2.5685451643563409E-10,65],
	[1.4205370182854874E-10,70],
	[8.8133228446069229E-11,75],
	[5.6536004817610585E-11,80],
	[3.5254393370690681E-11,85],
	[2.5369748675212485E-11,90]],"(N)"] call _interpolate,

	[[[-1.9546177766649412E-15,55],
	[-4.1539573979209121E-13,60],
	[-3.9823313153931106E-13,65],
	[-2.6524734529476917E-13,70],
	[-1.7440987934679887E-13,75],
	[-1.1083053016925233E-13,80],
	[-6.0742917624772107E-14,85],
	[-4.9466595349250448E-14,90]],"(O)"] call _interpolate
];


_return = [ _this select 0, _this select 1, _this select 2, [[_const_prone, _equation], [_const_stand, _equation]] ];
_return

